@@ -1,25 +1,36 @@
-# -*- coding: utf-8 -*-
-import six
-from django import VERSION
+from __future__ import annotations
+
+import copy
+from typing import Any, cast
+from collections.abc import Sequence
+
 from django import forms
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import fields
+from django.db.models import Model, fields
+from django.utils.encoding import force_str
+from django.utils.functional import Promise
+from django.utils.translation import override
 
 from modeltranslation import settings as mt_settings
+from modeltranslation.thread_context import fallbacks_enabled
 from modeltranslation.utils import (
-    get_language,
     build_localized_fieldname,
+    build_localized_intermediary_model,
     build_localized_verbose_name,
+    get_language,
     resolution_order,
 )
 from modeltranslation.widgets import ClearableWidgetWrapper
 
+from ._typing import Self
+from ._compat import is_hidden, clear_ForeignObjectRel_caches
 
 SUPPORTED_FIELDS = (
     fields.CharField,
     # Above implies also CommaSeparatedIntegerField, EmailField, FilePathField, SlugField
     # and URLField as they are subclasses of CharField.
     fields.TextField,
+    fields.json.JSONField,
     fields.IntegerField,
     # Above implies also BigIntegerField, SmallIntegerField, PositiveIntegerField and
     # PositiveSmallIntegerField, as they are subclasses of IntegerField.
@@ -36,10 +47,9 @@ SUPPORTED_FIELDS = (
     fields.files.ImageField,
     fields.related.ForeignKey,
     # Above implies also OneToOneField
+    fields.related.ManyToManyField,
 )
 
-NEW_RELATED_API = VERSION >= (1, 9)
-
 
 class NONE:
     """
@@ -51,7 +61,7 @@ class NONE:
     pass
 
 
-def create_translation_field(model, field_name, lang, empty_value):
+def create_translation_field(model: type[Model], field_name: str, lang: str, empty_value: Any):
     """
     Translation field factory. Returns a ``TranslationField`` based on a
     fieldname and a language.
@@ -64,27 +74,27 @@ def create_translation_field(model, field_name, lang, empty_value):
     If the class is neither a subclass of fields in ``SUPPORTED_FIELDS``, nor
     in ``CUSTOM_FIELDS`` an ``ImproperlyConfigured`` exception will be raised.
     """
-    if empty_value not in ('', 'both', None, NONE):
-        raise ImproperlyConfigured('%s is not a valid empty_value.' % empty_value)
-    field = model._meta.get_field(field_name)
+    if empty_value not in ("", "both", None, NONE):
+        raise ImproperlyConfigured("%s is not a valid empty_value." % empty_value)
+    field = cast(fields.Field, model._meta.get_field(field_name))
     cls_name = field.__class__.__name__
     if not (isinstance(field, SUPPORTED_FIELDS) or cls_name in mt_settings.CUSTOM_FIELDS):
-        raise ImproperlyConfigured('%s is not supported by modeltranslation.' % cls_name)
+        raise ImproperlyConfigured("%s is not supported by modeltranslation." % cls_name)
     translation_class = field_factory(field.__class__)
     return translation_class(translated_field=field, language=lang, empty_value=empty_value)
 
 
-def field_factory(baseclass):
-    class TranslationFieldSpecific(TranslationField, baseclass):
+def field_factory(baseclass: type[fields.Field]) -> type[TranslationField]:
+    class TranslationFieldSpecific(TranslationField, baseclass):  # type: ignore[valid-type, misc]
         pass
 
     # Reflect baseclass name of returned subclass
-    TranslationFieldSpecific.__name__ = 'Translation%s' % baseclass.__name__
+    TranslationFieldSpecific.__name__ = "Translation%s" % baseclass.__name__
 
     return TranslationFieldSpecific
 
 
-class TranslationField(object):
+class TranslationField:
     """
     The translation field functions as a proxy to the original field which is
     wrapped.
@@ -103,7 +113,14 @@ class TranslationField(object):
     that needs to be specified when the field is created.
     """
 
-    def __init__(self, translated_field, language, empty_value, *args, **kwargs):
+    def __init__(
+        self,
+        translated_field: fields.Field,
+        language: str,
+        empty_value: Any,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
         from modeltranslation.translator import translator
 
         # Update the dict of this field with the content of the original one
@@ -115,36 +132,41 @@ class TranslationField(object):
         self.language = language
         self.empty_value = empty_value
         if empty_value is NONE:
-            self.empty_value = None if translated_field.null else ''
+            self.empty_value = None if translated_field.null else ""
 
         # Default behaviour is that all translations are optional
         if not isinstance(self, fields.BooleanField):
             # TODO: Do we really want to enforce null *at all*? Shouldn't this
             # better honour the null setting of the translated field?
             self.null = True
+
+        # We preserve original blank value for translated fields,
+        # when they're in 'required_languages'.
+        # So they will be only required if original field itself was required.
+        original_blank = self.blank
         self.blank = True
 
-        # Take required_languages translation option into account
+        # Take required_languages translation option into account.
         trans_opts = translator.get_options_for_model(self.model)
         if trans_opts.required_languages:
             required_languages = trans_opts.required_languages
-            if isinstance(trans_opts.required_languages, (tuple, list)):
+            if isinstance(required_languages, (tuple, list)):
                 # All fields
                 if self.language in required_languages:
                     # self.null = False
-                    self.blank = False
+                    self.blank = original_blank
             else:
                 # Certain fields only
                 # Try current language - if not present, try 'default' key
                 try:
                     req_fields = required_languages[self.language]
                 except KeyError:
-                    req_fields = required_languages.get('default', ())
+                    req_fields = required_languages.get("default", ())
                 if self.name in req_fields:
                     # TODO: We might have to handle the whole thing through the
                     # FieldsAggregationMetaClass, as fields can be inherited.
                     # self.null = False
-                    self.blank = False
+                    self.blank = original_blank
 
         # Adjust the name of this field to reflect the language
         self.attname = build_localized_fieldname(self.translated_field.name, language)
@@ -157,28 +179,55 @@ class TranslationField(object):
         # (will show up e.g. in the admin).
         self.verbose_name = build_localized_verbose_name(translated_field.verbose_name, language)
 
-        # ForeignKey support - rewrite related_name
-        if not NEW_RELATED_API and self.rel and self.related and not self.rel.is_hidden():
-            import copy
+        if self.remote_field:
+            clear_ForeignObjectRel_caches(self.remote_field)
 
-            current = self.related.get_accessor_name()
-            self.rel = copy.copy(self.rel)  # Since fields cannot share the same rel object.
-            # self.related doesn't need to be copied, as it will be recreated in
-            # ``RelatedField.do_related_class``
+        # M2M support - <rewrite related_name> <patch intermediary model>
+        if isinstance(self.translated_field, fields.related.ManyToManyField) and hasattr(
+            self.remote_field, "through"
+        ):
+            # Since fields cannot share the same remote_field object:
+            self.remote_field = copy.copy(self.remote_field)
 
-            if self.rel.related_name is None:
-                # For implicit related_name use different query field name
-                loc_related_query_name = build_localized_fieldname(
-                    self.related_query_name(), self.language
+            # To support multiple relations to self, must provide a non null language scoped related_name
+            if self.remote_field.symmetrical and (
+                self.remote_field.model == "self"
+                or self.remote_field.model == self.model._meta.object_name
+                or self.remote_field.model == self.model
+            ):
+                self.remote_field.related_name = "%s_rel_+" % self.name
+            elif is_hidden(self.remote_field):
+                # Even if the backwards relation is disabled, django internally uses it, need to use a language scoped related_name
+                self.remote_field.related_name = "_%s_%s_+" % (
+                    self.model.__name__.lower(),
+                    self.name,
                 )
-                self.related_query_name = lambda: loc_related_query_name
-            self.rel.related_name = build_localized_fieldname(current, self.language)
-            self.rel.field = self  # Django 1.6
-            if hasattr(self.rel.to._meta, '_related_objects_cache'):
-                del self.rel.to._meta._related_objects_cache
-        elif NEW_RELATED_API and self.remote_field and not self.remote_field.is_hidden():
-            import copy
+            else:
+                # Default case with standard related_name must also include language scope
+                if self.remote_field.related_name is None:
+                    # For implicit related_name use different query field name
+                    loc_related_query_name = build_localized_fieldname(
+                        self.related_query_name(), self.language
+                    )
+                    self.related_query_name = lambda: loc_related_query_name
+                    self.remote_field.related_name = "%s_set" % (
+                        build_localized_fieldname(self.model.__name__.lower(), language),
+                    )
+                else:
+                    self.remote_field.related_name = build_localized_fieldname(
+                        self.remote_field.get_accessor_name(), language
+                    )
 
+            # Patch intermediary model with language scope to create correct db table
+            self.remote_field.through = build_localized_intermediary_model(
+                self.remote_field.through, language
+            )
+            self.remote_field.field = self
+
+            if hasattr(self.remote_field.model._meta, "_related_objects_cache"):
+                del self.remote_field.model._meta._related_objects_cache
+
+        elif self.remote_field and not is_hidden(self.remote_field):
             current = self.remote_field.get_accessor_name()
             # Since fields cannot share the same rel object:
             self.remote_field = copy.copy(self.remote_field)
@@ -190,8 +239,8 @@ class TranslationField(object):
                 )
                 self.related_query_name = lambda: loc_related_query_name
             self.remote_field.related_name = build_localized_fieldname(current, self.language)
-            self.remote_field.field = self  # Django 1.6
-            if hasattr(self.remote_field.model._meta, '_related_objects_cache'):
+            self.remote_field.field = self
+            if hasattr(self.remote_field.model._meta, "_related_objects_cache"):
                 del self.remote_field.model._meta._related_objects_cache
 
     # Django 1.5 changed definition of __hash__ for fields to be fine with hash requirements.
@@ -199,20 +248,33 @@ class TranslationField(object):
     # original field and fields didn't get added to sets.
     # So here we override __eq__ and __hash__ to fix the issue while retaining fine with
     # http://docs.python.org/2.7/reference/datamodel.html#object.__hash__
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, fields.Field):
             return self.creation_counter == other.creation_counter and self.language == getattr(
-                other, 'language', None
+                other, "language", None
             )
-        return super(TranslationField, self).__eq__(other)
+        return super().__eq__(other)
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash((self.creation_counter, self.language))
 
-    def formfield(self, *args, **kwargs):
+    def get_default(self) -> Any:
+        with override(self.language):
+            default = super().get_default()
+            # we must *force evaluation* at this point, otherwise the lazy translatable
+            # string is returned and will be evaluated only later when the main language
+            # is activated again (because this context block has exited).
+            #
+            # force_str passes protected types as-is, which includes None, int, float,
+            # datetime...
+            if isinstance(default, Promise):
+                default = force_str(default, strings_only=True)
+            return default
+
+    def formfield(self, *args: Any, **kwargs: Any) -> forms.Field:
         """
         Returns proper formfield, according to empty_values setting
         (only for ``forms.CharField`` subclasses).
@@ -235,29 +297,29 @@ class TranslationField(object):
         The ``forms.CharField`` somewhat surprising behaviour is documented as a
         "won't fix": https://code.djangoproject.com/ticket/9590.
         """
-        formfield = super(TranslationField, self).formfield(*args, **kwargs)
+        formfield = super().formfield(*args, **kwargs)
         if isinstance(formfield, forms.CharField):
             if self.empty_value is None:
                 from modeltranslation.forms import NullCharField
 
                 form_class = formfield.__class__
-                kwargs['form_class'] = type(
-                    'Null%s' % form_class.__name__, (NullCharField, form_class), {}
+                kwargs["form_class"] = type(
+                    "Null%s" % form_class.__name__, (NullCharField, form_class), {}
                 )
-                formfield = super(TranslationField, self).formfield(*args, **kwargs)
-            elif self.empty_value == 'both':
+                formfield = super().formfield(*args, **kwargs)
+            elif self.empty_value == "both":
                 from modeltranslation.forms import NullableField
 
                 form_class = formfield.__class__
-                kwargs['form_class'] = type(
-                    'Nullable%s' % form_class.__name__, (NullableField, form_class), {}
+                kwargs["form_class"] = type(
+                    "Nullable%s" % form_class.__name__, (NullableField, form_class), {}
                 )
-                formfield = super(TranslationField, self).formfield(*args, **kwargs)
+                formfield = super().formfield(*args, **kwargs)
                 if isinstance(formfield.widget, (forms.TextInput, forms.Textarea)):
                     formfield.widget = ClearableWidgetWrapper(formfield.widget)
         return formfield
 
-    def save_form_data(self, instance, data, check=True):
+    def save_form_data(self, instance: Model, data: Any, check: bool = True) -> None:
         # Allow 3rd-party apps forms to be saved using only translated field name.
         # When translated field (e.g. 'name') is specified and translation field (e.g. 'name_en')
         # not, we assume that form was saved without knowledge of modeltranslation and we make
@@ -268,55 +330,40 @@ class TranslationField(object):
         # Questionable fields are stored in special variable, which is later handled by clean_fields
         # method on the model.
         if check and self.language == get_language() and getattr(instance, self.name) and not data:
-            if not hasattr(instance, '_mt_form_pending_clear'):
+            if not hasattr(instance, "_mt_form_pending_clear"):
                 instance._mt_form_pending_clear = {}
             instance._mt_form_pending_clear[self.name] = data
         else:
-            super(TranslationField, self).save_form_data(instance, data)
+            super().save_form_data(instance, data)
 
-    def deconstruct(self):
+    def deconstruct(self) -> tuple[str, str, Sequence[Any], dict[str, Any]]:
         name, path, args, kwargs = self.translated_field.deconstruct()
         if self.null is True:
-            kwargs.update({'null': True})
-        if 'db_column' in kwargs:
-            kwargs['db_column'] = self.db_column
-        return six.text_type(self.name), path, args, kwargs
+            kwargs.update({"null": True})
+        if "db_column" in kwargs:
+            kwargs["db_column"] = self.db_column
+        return self.name, path, args, kwargs
 
-    def clone(self):
+    def clone(self) -> Self:
         from django.utils.module_loading import import_string
 
         name, path, args, kwargs = self.deconstruct()
         cls = import_string(path)
         return cls(*args, **kwargs)
 
-    def south_field_triple(self):
-        """
-        Returns a suitable description of this field for South.
-        """
-        # We'll just introspect the _actual_ field.
-        from south.modelsinspector import introspector
 
-        try:
-            # Check if the field provides its own 'field_class':
-            field_class = self.translated_field.south_field_triple()[0]
-        except AttributeError:
-            field_class = '%s.%s' % (
-                self.translated_field.__class__.__module__,
-                self.translated_field.__class__.__name__,
-            )
-        args, kwargs = introspector(self)
-        # That's our definition!
-        return (field_class, args, kwargs)
-
-
-class TranslationFieldDescriptor(object):
+class TranslationFieldDescriptor:
     """
     A descriptor used for the original translated field.
     """
 
     def __init__(
-        self, field, fallback_languages=None, fallback_value=NONE, fallback_undefined=NONE
-    ):
+        self,
+        field: fields.Field,
+        fallback_languages: dict[str, tuple[str, ...]] | None = None,
+        fallback_value: Any = NONE,
+        fallback_undefined: Any = NONE,
+    ) -> None:
         """
         Stores fallback options and the original field, so we know it's name
         and default.
@@ -334,7 +381,7 @@ class TranslationFieldDescriptor(object):
         instance.__dict__[self.field.name] = value
         if isinstance(self.field, fields.related.ForeignKey):
             instance.__dict__[self.field.get_attname()] = None if value is None else value.pk
-        if getattr(instance, '_mt_init', False) or getattr(instance, '_mt_disable', False):
+        if getattr(instance, "_mt_init", False) or getattr(instance, "_mt_disable", False):
             # When assignment takes place in model instance constructor, don't set value.
             # This is essential for only/defer to work, but I think it's sensible anyway.
             # Setting the localized field may also be disabled by setting _mt_disable.
@@ -371,7 +418,7 @@ class TranslationFieldDescriptor(object):
             val = getattr(instance, loc_field_name, None)
             if self.meaningful_value(val, undefined):
                 return val
-        if mt_settings.ENABLE_FALLBACKS and self.fallback_value is not NONE:
+        if fallbacks_enabled() and self.fallback_value is not NONE:
             return self.fallback_value
         else:
             if default is NONE:
@@ -387,13 +434,15 @@ class TranslationFieldDescriptor(object):
             return default
 
 
-class TranslatedRelationIdDescriptor(object):
+class TranslatedRelationIdDescriptor:
     """
     A descriptor used for the original '_id' attribute of a translated
     ForeignKey field.
     """
 
-    def __init__(self, field_name, fallback_languages):
+    def __init__(
+        self, field_name: str, fallback_languages: dict[str, tuple[str, ...]] | None
+    ) -> None:
         self.field_name = field_name  # The name of the original field (excluding '_id')
         self.fallback_languages = fallback_languages
 
@@ -420,24 +469,24 @@ class TranslatedRelationIdDescriptor(object):
         return None
 
 
-class LanguageCacheSingleObjectDescriptor(object):
+class TranslatedManyToManyDescriptor:
     """
-    A Mixin for RelatedObjectDescriptors which use current language in cache lookups.
+    A descriptor used to return correct related manager without language fallbacks.
     """
 
-    accessor = None  # needs to be set on instance
+    def __init__(
+        self, field_name: str, fallback_languages: dict[str, tuple[str, ...]] | None
+    ) -> None:
+        self.field_name = field_name  # The name of the original field
+        self.fallback_languages = fallback_languages
 
-    @property
-    def cache_name(self):
-        """
-        Used in django 1.x
-        """
-        lang = get_language()
-        cache = build_localized_fieldname(self.accessor, lang)
-        return "_%s_cache" % cache
+    def __get__(self, instance, owner):
+        # TODO: do we really need to handle fallbacks with m2m relations?
+        loc_field_name = build_localized_fieldname(self.field_name, get_language())
+        loc_attname = (instance or owner)._meta.get_field(loc_field_name).get_attname()
+        return getattr((instance or owner), loc_attname)
 
-    def get_cache_name(self):
-        """
-        Used in django > 2.x
-        """
-        return build_localized_fieldname(self.accessor, get_language())
+    def __set__(self, instance, value):
+        loc_field_name = build_localized_fieldname(self.field_name, get_language())
+        loc_attname = instance._meta.get_field(loc_field_name).get_attname()
+        setattr(instance, loc_attname, value)
